"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildFunctions = exports.getOutFileForLambda = exports.getOutDirForLambda = void 0;
const esbuild_1 = require("esbuild");
const esbuild_plugin_pnp_1 = require("@yarnpkg/esbuild-plugin-pnp");
const esbuild_css_modules_server_plugin_1 = __importDefault(require("esbuild-css-modules-server-plugin"));
const generateFunctionName_1 = require("./generate/generateFunctionName");
const utils_sh_1 = require("@goldstack/utils-sh");
const getOutDirForLambda = (config) => {
    if (config.path === '$default') {
        return `./distLambda/${config.path}`;
    }
    if (config.path.endsWith('/')) {
        return `./distLambda${config.path}index`;
    }
    return `./distLambda${config.path}`;
};
exports.getOutDirForLambda = getOutDirForLambda;
const getOutFileForLambda = (config) => {
    return `${(0, exports.getOutDirForLambda)(config)}/lambda.js`;
};
exports.getOutFileForLambda = getOutFileForLambda;
const buildFunctions = async ({ routesDir, configs, lambdaNamePrefix, }) => {
    const esbuildConfig = (0, utils_sh_1.readToType)('./esbuild.config.json');
    await (0, utils_sh_1.rmSafe)('./distLambda');
    (0, utils_sh_1.mkdir)('-p', './distLambda/zips');
    for await (const config of configs) {
        (0, utils_sh_1.mkdir)('-p', (0, exports.getOutDirForLambda)(config));
        const esbuildLocalPath = (0, utils_sh_1.changeExtension)(`${routesDir}/${config.relativeFilePath}`, '.esbuild.config.json');
        const functionName = (0, generateFunctionName_1.generateFunctionName)(lambdaNamePrefix, config);
        const localEsbuildConfig = (0, utils_sh_1.readToType)(esbuildLocalPath);
        const generatedCss = [];
        const res = await (0, esbuild_1.build)({
            plugins: [
                (0, esbuild_css_modules_server_plugin_1.default)({
                    onCSSGenerated: (css) => {
                        generatedCss.push(css);
                    },
                }),
                (0, esbuild_plugin_pnp_1.pnpPlugin)(),
            ],
            bundle: true,
            entryPoints: [`${routesDir}/${config.relativeFilePath}`],
            external: [
                'aws-sdk', // included in Lambda runtime environment
            ],
            minify: true,
            platform: 'node',
            format: 'cjs',
            target: 'node16.0',
            treeShaking: true,
            define: {
                'process.env.NODE_ENV': '"production"',
            },
            sourcemap: true,
            outfile: (0, exports.getOutFileForLambda)(config),
            metafile: true,
            ...esbuildConfig,
            ...localEsbuildConfig,
        });
        if (!res.metafile) {
            throw new Error(`Metafile for ${functionName} not defined.`);
        }
        (0, utils_sh_1.write)(JSON.stringify(res.metafile), `./distLambda/zips/${functionName}.meta.json`);
        // provide CSS for initial load
        (0, utils_sh_1.write)(generatedCss.join('\n'), `${(0, exports.getOutDirForLambda)(config)}/client.bundle.css`);
    }
};
exports.buildFunctions = buildFunctions;
//# sourceMappingURL=buildFunctions.js.map